@@ -1,5 +1,5 @@
 package PAUSE::Permissions;
-$PAUSE::Permissions::VERSION = '0.11';
+$PAUSE::Permissions::VERSION = '0.12';
 use strict;
 use warnings;
 
@@ -8,10 +8,13 @@ use PAUSE::Permissions::Module;
 use PAUSE::Permissions::ModuleIterator;
 use PAUSE::Permissions::EntryIterator;
 use File::HomeDir;
-use File::Spec::Functions 'catfile';
-use HTTP::Date qw(time2str);
+
+use File::Spec::Functions qw/ catfile  /;
+use HTTP::Date            qw/ time2str / ;
+use Carp                  qw/ croak    /;
+use Time::Duration::Parse qw/ parse_duration /;
+
 use HTTP::Tiny;
-use Carp;
 
 my $DISTNAME = 'PAUSE-Permissions';
 my $BASENAME = '06perms.txt';
@@ -22,21 +25,54 @@ has 'url' =>
      default => sub { return 'http://www.cpan.org/modules/06perms.txt'; },
     );
 
-has 'path' =>
-    (
-     is => 'rw',
-    );
+has 'path'         => (is => 'ro' );
+has 'cache_path'   => (is => 'lazy' );
+has 'max_age'      => (is => 'ro');
+has 'preload'      => (is => 'ro', default => sub { 0 });
+has 'module_cache' => (is => 'lazy');
+
+sub _build_cache_path
+{
+    my $self     = shift;
+
+    my $basename = $self->url;
+       $basename =~ s!^.*[/\\]!!;
+    my $classid  = ref($self);
+       $classid  =~ s/::/-/g;
+
+    return catfile(File::HomeDir->my_dist_data( $classid, { create => 1 } ), $basename);
+}
+
+sub _build_module_cache
+{
+    my $self     = shift;
+    my $iterator = $self->module_iterator;
+    my $cache    = {};
+
+    while (my $module = $iterator->next_module) {
+        $cache->{ $module->name } = $module;
+    }
+
+    return $cache;
+}
 
 sub BUILD
 {
     my $self = shift;
 
-    # If constructor didn't specify a local file, then mirror the file from CPAN
-    if (not $self->path) {
-        $self->path( catfile(File::HomeDir->my_dist_data( $DISTNAME, { create => 1 } ), $BASENAME) );
-        # HTTP::Tiny->new()->mirror($self->url, $self->path);
-        $self->_cache_file_if_needed();
+    if ($self->path) {
+        return if -f $self->path;
+        croak "the file you specified with 'path' doesn't exist";
     }
+
+    # If we already have a locally cached copy, and the max_age was specified,
+    # then check if our cache has expired
+    if (-f $self->cache_path && $self->max_age) {
+        my $max_age_in_seconds = parse_duration($self->max_age);
+        return unless time() - $max_age_in_seconds > (stat($self->cache_path))[9];
+    }
+
+    $self->_cache_file_if_needed();
 }
 
 sub _cache_file_if_needed
@@ -45,8 +81,8 @@ sub _cache_file_if_needed
     my $options = {};
     my $ua      = HTTP::Tiny->new();
 
-    if (-f $self->path) {
-        $options->{'If-Modified-Since'} = time2str( (stat($self->path))[9]);
+    if (-f $self->cache_path) {
+        $options->{'If-Modified-Since'} = time2str( (stat($self->cache_path))[9]);
     }
     my $response = $ua->get($self->url, $options);
 
@@ -78,7 +114,7 @@ sub _transform_and_cache
         push(@lines, [lc($package), lc($user), $package, $user, $perm]);
     }
 
-    open(my $fh, '>', $self->path);
+    open(my $fh, '>', $self->cache_path);
     print $fh <<'END_HEADER';
 File: PAUSE Permissions data
 Format: 2
@@ -107,6 +143,14 @@ sub module_iterator
     return PAUSE::Permissions::ModuleIterator->new( permissions => $self );
 }
 
+sub open_file
+{
+    my $self     = shift;
+    my $filename = defined($self->path) ? $self->path : $self->cache_path;
+    open(my $fh, '<', $filename) || croak "can't open $filename: $!";
+    return $fh;
+}
+
 sub module_permissions
 {
     my $self   = shift;
@@ -118,8 +162,11 @@ sub module_permissions
     my %perms;
     my ($m, $u, $p);
 
-    open($fh, '<', $self->path)
-        || die "can't read local file ", $self->path, ": $!\n";
+    if ($self->preload && $self->module_cache) {
+        return $self->module_cache->{$module} // undef;
+    }
+
+    $fh = $self->open_file();
     while (<$fh>) {
         chomp;
         if ($inheader && /^\s*$/) {
@@ -158,7 +205,7 @@ PAUSE::Permissions - interface to PAUSE's module permissions file (06perms.txt)
 
   use PAUSE::Permissions 0.08;
   
-  my $pp = PAUSE::Permissions->new;
+  my $pp = PAUSE::Permissions->new(max_age => '1 day');
   my $mp = $pp->module_permissions('HTTP::Client');
   
   my $owner    = $mp->owner;
@@ -179,7 +226,10 @@ The format and interpretation of this file
 are covered in L</"The 06perms.txt file"> below.
 
 By default, the module will mirror C<06perms.txt> from CPAN,
-using L<HTTP::Tiny> to request it and store it locally.
+using L<HTTP::Tiny> to request it and store it locally
+What gets cached locally is actually a transformed version of 06perms.txt
+for easier processing.
+
 By default it will get the file from L<http://www.cpan.org>, but you can
 pass an alternate URI to the constructor:
 
@@ -188,7 +238,10 @@ pass an alternate URI to the constructor:
 
 If you've already got a copy lying around, you can tell the module to use that:
 
-  $pp = PAUSE::Permissions->new( path => '/tmp/06perms.txt' );
+  $pp = PAUSE::Permissions->new( path => '/tmp/my06perms.txt' );
+
+Note that the file you provide this way must be in the post-processed
+format, and not a raw copy of C<06perms.txt>.
 
 Having created an instance of C<PAUSE::Permissions>,
 you can then call the C<module_permissions> method
@@ -199,6 +252,28 @@ B<Note>: you should make sure you're using version 0.08 or later.
 PAUSE now treats package names case insensitively with respect to
 permissions, so this module does now as well.
 
+=head2 Getting permissions for multiple modules
+
+Sometimes you might want to use the C<module_permissions()> method
+to get permissions for multiple modules, for example if you've built
+up a list of modules from elsewhere. If you're doing this, then you
+should set the C<preload> attribute to a true value:
+
+ use PAUSE::Permissions 0.12;
+
+ my $pp = PAUSE::Permissions->new(preload => 1);
+ foreach my $module_name (@long_list_of_modules) {
+    my $mp = $pp->module_permissions($module_name);
+    # do something with $mp (instance of PAUSE::Permissions::Module)
+ }
+
+With the C<preload> option enabled, the permissions data for I<all>
+modules will be pre-loaded into memory, making the above code much
+quicker, trading that off against the memory used.
+
+This attribute was introduced in version 0.12, so you should
+specify the minimum version when C<use>'ing C<PAUSE::Permission>.
+
 =head1 METHODS
 
 There are only four methods you need to know:
@@ -215,17 +290,36 @@ The constructor takes a hash of options:
 
 =item *
 
-B<path>: the path to a local copy of 06perms.txt.
+B<cache_path>: the full path to the location where you'd like
+C<PAUSE::Permissions> to cache the transformed version of 06perms.txt.
+
+=item *
+
+B<path>: your own local copy of the file, to use instead of the
+version in the C<cache_path>.
+Note that this must be in the post-processed format for the local cache,
+and not the original raw format of C<06perms.txt>.
+
 The constructor will C<die()> if the file doesn't exist, or isn't readable.
-If you don't provide this parameter, then we'll try and get 06perms.txt
-from the C<url> parameter, and store it in a local directory,
-determined by C<File::HomeDir-E<gt>my_dist_data>.
 
 =item *
 
 B<url>: the URL for 06perms.txt;
 defaults to L<http://www.cpan.org/modules/06perms.txt>
 
+=item *
+
+B<max_age>: the expiration time for cached data, once C<06perms.txt> has been grabbed.
+The age can be specified using any format supported by L<Time::Duration::Parse>,
+such '1 day', '2 minutes and 30 seconds', or '02:30:00'.
+
+=item *
+
+B<preload>: load all module permissions data into memory,
+to speed up repeated calls to C<module_permissions()>.
+This currently (0.12 onwards) doesn't currently affect any
+other methods, though it might in a future release.
+
 =back
 
 So you might use the following,
@@ -479,11 +573,6 @@ once I've got the details for the requested module. Would be a lot more efficien
 to keep the file open and start the search from there, as the file is sorted.
 A binary chop on the file would be much more efficient as well.
 
-=item *
-
-The 06perms.txt file is currently mirrored with an If-Modified-Since request.
-We should probably also support a mechanism for saying things like "only get it if
-my copy is more than N days old". And consider using rsync as well.
 
 =item *
 
